% sym_contour.m
% plots contours and mesh plots of the ratio of the energy
% in the asymmetric to symmetric parts of the solution.
% the independent variables are the sizes of the CP and TEQ.
%
% Written by Rick Martin, 07/31/2002
% Last updated 08/26/2002
% Cornell University Blind Equalization Research Group

% DMT TEQ toolbox from U.T. Austin
addpath ../../bevans/teqv3

% Miscellaneous functions
addpath ../mcmtools

% Note: the paper uses only nu = 32, and produces 1 plot.
%   Change the range of nu below to see mesh and contour plots
%   with nu and Lw as the independent variables.
%
% The plots are computed for both the MSSNR and MSSNR-UNT designs.
%
% This code takes a very long time to run, especially
%   for large ranges of values of Lw and nu.

% system parameters
Lws = [3:1:200]-1;    % equalizer lengths - 1
%nus = [1:1:140];      % cyclic prefix lengths
nus = 32;             % cyclic prefix length
Nlw = length(Lws);
Nnu = length(nus);
Ncsa = 8;             % average over CSA loops 1 to Ncsa

% allocate memory
Rmel = zeros(Nlw,Nnu,Ncsa);
Runt = zeros(Nlw,Nnu,Ncsa);

% loop through channel, TEQ length, and CP length
hh = waitbar(0,'Computing many TEQs...');
for k=1:Ncsa
 k
 % load CSA channels from U.T. Austin's toolbox
 eval(['load csaloop',num2str(k),'.time;']);
 eval(['h = (csaloop',num2str(k),'(:,2));']);
 %h = randn(75,1);
 h = h/norm(h);      % unit norm channel
 Lh = length(h)-1;   % channel length - 1
for i=1:Nlw
   Lw = Lws(i);      % equalizer length - 1
   Lc = Lw+Lh;       % length of combined channel - 1
   for j=1:Nnu
      nu = nus(j);   % cyclic prefix length

      % maximum shortening SNR solution from UT Austin's toolbox
      [wmelsa,dopt_mel,dummy] = mssnr(h,nu+1,Lw+1,1,Lc-nu-1,0);
      cmel1 = conv(h,wmelsa).';
      %cmelsa = cmel1/norm(cmel1);   % scale it to unit norm
      delta = dopt_mel;   % pick delay for fair comparison

      % MSSNR solution with unit norm TEQ (w_unt)
      H = convmtx(h,Lw+1);
      Hwin = H([delta+1:delta+nu+1],:);
      Hwall = H([1:delta,delta+nu+2:end],:);
      A = Hwall'*Hwall;
      B = Hwin'*Hwin;
      [V,D] = eig(A);
      [dummy,index] = min(abs(diag(D)));
      w_unt = V(:,index);
      if Nlw == 1 & Nnu == 1, c_unt = conv(h,w_unt); end;

      % decompose TEQ into a symmetric and an asymmetric part
      [w_unt_s,w_unt_a] = symasym(w_unt);
      [w_mel_s,w_mel_a] = symasym(wmelsa);

      % find Energy ratios of asymmetric to symmetric part
      Rmel(i,j,k) = (norm(w_mel_a)/norm(w_mel_s))^2;
      Runt(i,j,k) = (norm(w_unt_a)/norm(w_unt_s))^2;

   end
   waitbar( ((k-1)*Nlw+i)/(Ncsa*Nlw), hh);
end
end
close(hh);

aRmel = mean(Rmel,3);
aRunt = mean(Runt,3);

if Nnu > 1

levels = [0.005:0.005:0.3];
levels2 = [0.001:0.001:0.3];

figure(1)
clf
set(0,'DefaultAxesFontSize',16);
contour(Lws+1,nus,aRmel.',levels);
title('Contours of ||w_{skew}||^2/||w_{sym}||^2 for MSSNR TEQ',...
   'FontSize',16);
xlabel('length of TEQ','FontSize',16);
ylabel('length of CP','FontSize',16);
colorbar

figure(2)
clf
set(0,'DefaultAxesFontSize',16);
contour(Lws+1,nus,aRunt.',levels2);
title('Contours of ||w_{skew}||^2/||w_{sym}||^2 for unit norm TEQ',...
   'FontSize',16);
xlabel('length of TEQ','FontSize',16);
ylabel('length of CP','FontSize',16);
colorbar

figure(3);
clf
set(0,'DefaultAxesFontSize',16);
mesh(Lws+1,nus,min(aRmel.',1));
title('||w_{skew}||^2/||w_{sym}||^2 for MSSNR TEQ','FontSize',16);
xlabel('length of TEQ','FontSize',16); 
ylabel('length of CP','FontSize',16);

figure(4);
clf
set(0,'DefaultAxesFontSize',16);
mesh(Lws+1,nus,aRunt.');
title('||w_{skew}||^2/||w_{sym}||^2 for MSSNR-UEC TEQ','FontSize',16);
xlabel('length of TEQ','FontSize',16);
ylabel('length of CP','FontSize',16);

elseif Nnu == 1

figure(1)
clf
set(0,'DefaultAxesFontSize',16);
plot(Lws+1,aRmel,'b-', Lws+1,aRunt,'r--');
%title('','FontSize',16);
xlabel('length of TEQ','FontSize',16);
ylabel('||w_{skew}||^2/||w_{sym}||^2','FontSize',16);
legend('MSSNR','MSSNR-UNT');

end
