% sym_teq.m
% plots TEQ, symmetric part, and asymmetric part
%
% Written by Rick Martin, 07/31/2002
% Last updated 08/21/2002
% Cornell University Blind Equalization Research Group

% DMT TEQ toolbox
addpath ../../bevans/teqv3

% Miscellaneous functions
addpath ../mcmtools

% system parameters
Lw = 50 - 1;       % equalizer length - 1
nu = 32;            % cyclic prefix length
load ../j02_merry/channels/csaloop1.time;
h = (csaloop1(:,2));
%h = randn(75,1);
h = h/norm(h);      % unit norm channel
Lh = length(h)-1;   % channel length - 1
Lc = Lw+Lh;         % length of combined channel - 1

% set delay (d)
% use d = -1 to use the best d from the MSSNR problem
d = -1;
if d == -1
   dmin = 1; dmax = Lc-nu-1;
else
   dmin = d; dmax = d;
end

% maximum shortening SNR solution from UT Austin's toolbox
[wmelsa,dopt_mel,dummy] = mssnr(h,nu+1,Lw+1,dmin,dmax,0);
cmel1 = conv(h,wmelsa).';
c_mel = cmel1/norm(cmel1);   % scale it to unit norm
delta = dopt_mel;   % pick delay for fair comparison

% MSSNR solution with unit norm TEQ (w_unt)
H = convmtx(h,Lw+1);
Hwin = H([delta+1:delta+nu+1],:);
Hwall = H([1:delta,delta+nu+2:end],:);
A = Hwall'*Hwall;
B = Hwin'*Hwin;
[V,D] = eig(A);
[dummy,index] = min(abs(diag(D)));
w_unt = V(:,index);
c_unt = conv(h,w_unt);

% decompose A and B into symmetric and skew-symmetric parts
J = fliplr(eye(size(A)));
Ac = J*A*J;
Asym = (A+Ac)/2;
Askew = (A-Ac)/2;
Bc = J*B*J;
Bsym = (B+Bc)/2;
Bskew = (B-Bc)/2;
% ratio of squares of Frobenius norms
FA = sum(diag(Askew'*Askew)) / sum(diag(Asym'*Asym))
FB = sum(diag(Bskew'*Bskew)) / sum(diag(Bsym'*Bsym))

% decompose TEQ into a symmetric and an asymmetric (skew) part
[w_unt_s,w_unt_a] = symasym(w_unt);
[w_mel_s,w_mel_a] = symasym(wmelsa);
c_unt_a = conv(h,w_unt_a);
c_unt_s = conv(h,w_unt_s);
c_mel_a = conv(h,w_mel_a);
c_mel_s = conv(h,w_mel_s);

% find Energy ratios of asymmetric to symmetric part
Rmel = (norm(w_mel_a)/norm(w_mel_s))^2
Runt = (norm(w_unt_a)/norm(w_unt_s))^2

% find TIR (b) for w_unt
b_unt = c_unt([delta+1:delta+nu+1]);

%-------------------------------------------------------
% plot equalizer decomposition for unit norm TEQ
figure(1);
clf
set(0,'DefaultAxesFontSize',16);
subplot(3,1,1);
stem([0:Lw],w_unt);
ylabel('full TEQ','FontSize',16);
title('TEQ taps for ||w|| = 1','FontSize',16);
v = axis;
grid on
subplot(3,1,2);
stem([0:Lw],w_unt_s);
ylabel('w_{sym}','FontSize',16);
axis([v(1) v(2) v(3) v(4)]);
grid on
subplot(3,1,3);
stem([0:Lw],w_unt_a);
ylabel('w_{skew}','FontSize',16);
axis([v(1) v(2) v(3) v(4)]);
grid on

% plot equalizer decomposition for unit norm window
figure(2);
clf
set(0,'DefaultAxesFontSize',16);
subplot(3,1,1);
stem([0:Lw],wmelsa,'filled');
ylabel('full TEQ','FontSize',16)
title('TEQ taps for ||c_{win}|| = 1','FontSize',16);
v = axis;
grid on
subplot(3,1,2);
stem([0:Lw],w_mel_s,'filled');
ylabel('w_{sym}','FontSize',16);
axis([v(1) v(2) v(3) v(4)]);
grid on
subplot(3,1,3);
stem([0:Lw],w_mel_a,'filled');
axis([v(1) v(2) v(3) v(4)]);
ylabel('w_{skew}','FontSize',16);
grid on

% plot combined channels for unit norm TEQ   
figure(3);
clf
set(0,'DefaultAxesFontSize',16);
subplot(3,1,1);
plot([0:Lc],c_unt,...
   [0:Lh],h*max(abs(c_unt))/max(abs(h)),'r--');
ylabel('channel+TEQ','FontSize',16);
title('Combined channel for ||w|| = 1','FontSize',16);
%v = axis;
axis([0 200 min(c_unt) max(c_unt)]);
grid on
subplot(3,1,2);
plot([0:Lc],c_unt_s);
ylabel('c_{sym}','FontSize',16);
axis([0 200 min(c_unt_s) max(c_unt_s)]);
grid on
subplot(3,1,3);
plot([0:Lc],c_unt_a);
ylabel('c_{skew}','FontSize',16);
axis([0 200 min(c_unt_a) max(c_unt_a)]);
grid on

% plot combined channels for unit norm window
figure(4);
clf
set(0,'DefaultAxesFontSize',16);
subplot(3,1,1);
plot([0:Lc],c_mel);
ylabel('full TEQ','FontSize',16)
title('Combined channel for ||c_{win}|| = 1','FontSize',16);
%v = axis;
axis([0 200 min(c_mel) max(c_mel)]);
grid on
subplot(3,1,2);
plot([0:Lc],c_mel_s);
ylabel('c_{sym}','FontSize',16);
axis([0 200 min(c_mel_s) max(c_mel_s)]);
grid on
subplot(3,1,3);
plot([0:Lc],c_mel_a);
axis([0 200 min(c_mel_a) max(c_mel_a)]);
ylabel('c_{skew}','FontSize',16);
grid on

% plot magnitude response of MSSNR-TEQ
th = [0:.01:2.1*pi];
rw = roots(w_unt);
rh = roots(h);
rb = roots(b_unt);
%
figure(5);
clf
set(0,'DefaultAxesFontSize',16);
%freqz(w_unt);
plot(cos(th),sin(th),'k-', real(rh),imag(rh),'r+',...
   real(rw),imag(rw),'bo', real(rb),imag(rb),'kx');
axis image;
legend('unit circle','channel roots',...
   'TEQ roots','TIR roots',-1);

if 0

figure(1); print -depsc symteq_mel1.eps
figure(2); print -depsc symteq_unt1.eps

end
